import { getCanvasPoint } from './core';
export function subscriber(params) {
    const { canvas } = params;
    return (event, func, callback) => {
        const handler = callback && func && func(params);
        if (handler)
            canvas.addEventListener(event, handler);
        return () => handler && canvas.removeEventListener(event, handler);
    };
}
function clickHandler(params) {
    const canvas = params.canvas;
    const { ratio, onClick } = params.options;
    return (event) => {
        if (!onClick)
            return;
        const ctx = canvas.getContext('2d');
        const [cX, cY] = getCanvasPoint(canvas, ratio, event);
        for (let i = 0; i < params.drawData.length; i++) {
            const { mask, data } = params.drawData[i];
            if (ctx.isPointInPath(mask, cX, cY)) {
                onClick(data);
                return;
            }
        }
    };
}
function moveHandler(drawFunc, getData) {
    return (params) => {
        const canvas = params.canvas;
        const ctx = canvas.getContext('2d');
        const { ratio, onClick, onHoverChange } = params.options;
        return (event) => {
            if (!onHoverChange)
                return;
            let cursor = 'default';
            let found;
            const [cX, cY] = getCanvasPoint(canvas, ratio, event);
            drawFunc(params);
            const { items, fill } = getData();
            for (let i = 0; i < items.length; i++) {
                const { data, mask } = items[i];
                if (ctx.isPointInPath(mask, cX, cY)) {
                    ctx.fillStyle = fill(data.color);
                    ctx.fill(mask);
                    cursor = 'pointer';
                    found = data;
                    break;
                }
            }
            const { clientX, clientY } = event;
            onHoverChange(found && { data: found, clientX, clientY });
            if (onClick)
                canvas.style.cursor = cursor;
        };
    };
}
function leaveHandler(drawFunc) {
    return (params) => {
        const canvas = params.canvas;
        const { onClick, onHoverChange } = params.options;
        return () => {
            drawFunc(params);
            if (onHoverChange)
                onHoverChange(undefined);
            if (onClick)
                canvas.style.cursor = 'default';
        };
    };
}
export function setupEvents(params, drawFunc, getData) {
    const sub = subscriber(params);
    const { onClick, onHoverChange } = params.options;
    const unsubClick = sub('click', clickHandler, onClick);
    const unsubMove = sub('mousemove', moveHandler(drawFunc, getData), onHoverChange);
    const unsubLeave = sub('mouseleave', leaveHandler(drawFunc), onHoverChange);
    return () => {
        unsubClick();
        unsubMove();
        unsubLeave();
    };
}
//# sourceMappingURL=events.js.map